
# Environment variables names:

PERMEDCOE_TMPDIR = "PERMEDCOE_TMPDIR"
PERMEDCOE_PROCESSES = "PERMEDCOE_PROCESSES"
PERMEDCOE_GPUS = "PERMEDCOE_GPUS"
PERMEDCOE_MEMORY = "PERMEDCOE_MEMORY"
PERMEDCOE_MOUNT_POINTS = "PERMEDCOE_MOUNT_POINTS"

# Global variables:

SEPARATOR = "------------------------------------------"



import logging

LOG_LEVEL_DEBUG = "debug"
LOG_LEVEL_INFO = "info"
LOG_LEVEL_WARNING = "warning"
LOG_LEVEL_ERROR = "error"
LOG_LEVEL_CRITICAL = "critical"

LOGGING_FORMAT = "%(asctime)s - %(levelname)s - %(message)s"


def init_logging(debug=False, level=LOG_LEVEL_ERROR):
    """ Initialize logging.
    Supported levels: ["debug" | "info" | "warning" | "error" | "critical"]

    Args:
        debug (bool, optional): Force debug mode.
        level (str, optional): Log level. Defaults to LOG_LEVEL_ERROR.
    """
    if debug:
        log_level = logging.DEBUG
    else:
        # Select the logging level
        if level == LOG_LEVEL_CRITICAL:
            log_level = logging.CRITICAL
        elif level == LOG_LEVEL_ERROR:
            log_level = logging.ERROR
        elif level == LOG_LEVEL_WARNING:
            log_level = logging.WARNING
        elif level == LOG_LEVEL_INFO:
            log_level = logging.INFO
        else:
            log_level = logging.DEBUG
    # Initialize logging
    logging.basicConfig(level=log_level, format=LOGGING_FORMAT)



# ########################## #
# ####### PUBLIC API ####### #
# ########################## #

# Decorator selector
try:
    # Running with PyCOMPSs with take these imports
    from pycompss.api.container import container
    from pycompss.api.constraint import constraint
    from pycompss.api.binary import binary
    from pycompss.api.task import task
    from pycompss.api.mpi import mpi
    from pycompss.api.parameter import FILE_IN
    from pycompss.api.parameter import FILE_OUT
    from pycompss.api.parameter import FILE_INOUT
    from pycompss.api.parameter import DIRECTORY_IN
    from pycompss.api.parameter import DIRECTORY_OUT
    from pycompss.api.parameter import DIRECTORY_INOUT
    from pycompss.api.parameter import Type
    from pycompss.api.parameter import StdIOStream
    from pycompss.api.parameter import STDIN
    # raise ImportError  # NOSONAR
except ImportError:
    # Without PyCOMPSs it will take the core
    from permedcoe.core.decorators import container
    from permedcoe.core.decorators import constraint
    from permedcoe.core.decorators import binary
    from permedcoe.core.decorators import task
    from permedcoe.core.decorators import mpi
    from permedcoe.core.decorators import FILE_IN
    from permedcoe.core.decorators import FILE_OUT
    from permedcoe.core.decorators import FILE_INOUT
    from permedcoe.core.decorators import DIRECTORY_IN
    from permedcoe.core.decorators import DIRECTORY_OUT
    from permedcoe.core.decorators import DIRECTORY_INOUT
    from permedcoe.core.decorators import Type
    from permedcoe.core.decorators import StdIOStream
    from permedcoe.core.decorators import STDIN

# Public functions
from permedcoe.base import get_environment
from permedcoe.base import set_debug
from permedcoe.base import invoker

# Arguments definition - explicit arguments
from permedcoe.utils.user_arguments import Arguments



#!/usr/bin/python3

# Decorator imports
from permedcoe import constraint       # To define constraints needs (e.g. number of cores)
from permedcoe import container        # To define container related needs
from permedcoe import binary           # To define binary to execute related needs
from permedcoe import mpi              # To define an mpi binary to execute related needs (can not be used with @binary)
from permedcoe import task             # To define task related needs
# @task supported types
from permedcoe import FILE_IN          # To define file type and direction
from permedcoe import FILE_OUT         # To define file type and direction
from permedcoe import FILE_INOUT       # To define file type and direction
from permedcoe import DIRECTORY_IN     # To define directory type and direction
from permedcoe import DIRECTORY_OUT    # To define directory type and direction
from permedcoe import DIRECTORY_INOUT  # To define directory type and direction
# Other permedcoe available functionalities
from permedcoe import Arguments        # Arguments definition
from permedcoe import get_environment  # Get variables from invocation (tmpdir, processes, gpus, memory)

# Import single container and assets definitions
from NEW_NAME.definitions import NEW_NAME_ASSETS_PATH  # binary could be in this folder
from NEW_NAME.definitions import NEW_NAME_CONTAINER
from NEW_NAME.definitions import COMPUTING_UNITS


def function_name(*args, **kwargs):
    """Extended python interface:
    To be used only with PyCOMPSs - Enables to define a workflow within the building block.
    Tasks are not forced to be binaries: PyCOMPSs supports tasks that are pure python code.

    # PyCOMPSs help: https://pycompss.readthedocs.io/en/latest/Sections/02_App_Development/02_Python.html

    Requirement: all tasks should be executed in a container (with the same container definition)
                 to ensure that they all have the same requirements.
    """
    print("Building Block entry point to be used with PyCOMPSs")
    # TODO: (optional) Pure python code calling to PyCOMPSs tasks (that can be defined in this file or in another).


@container(engine="SINGULARITY", image=NEW_NAME_CONTAINER)
@binary(binary="cp")                                        # TODO: Define the binary to be used (can be within NEW_NAME_ASSETS_PATH).
@task(input_file=FILE_IN, output_file=FILE_OUT)             # TODO: Define the inputs and output parameters.
def building_block_task(input_file=None,                    # TODO: Define a representative task name
                        output_file=None,
                        verbose="-v"):                      # TODO: Define the binary parameters
    """Summary.

    The Definition is equal to:
       cp <input_file> <output_file> -v
    Empty function since it represents a binary execution:

    :param input_file: Input file description, defaults to None
    :type input_file: str, optional
    :param verbose: Verbose description, defaults to "-v"
    :type verbose: str, optional
    """
    pass


def invoke(input, output, config):
    """Common interface.

    Args:
        input (str): Input file path.
        output (str): Output directory path.
        config (dict): Configuration dictionary.
    Returns:
        None
    """
    # TODO: Define the arguments required by the Building Block in definition.json file

    # TODO: Declare how to run the binary specification (convert config into building_block_task call)
    # Sample config parameter get:
    #     operation = config["operation"]
    # Then operation can be used to tune the building_block_task parameters or even be a parameter.
    # Sample permedcoe environment get:
    #     env_vars = get_environment()
    # Retrieves the extra flags from permedcoe.
    input_file = input[0]
    output_file = output[0]
    building_block_task(input_file=input_file,
                        output_file=output_file)



import os
from permedcoe.bb import CONTAINER_PATH
from permedcoe.bb import COMPUTING_UNITS

# Do not change this line
BB_SOURCE_PATH=os.path.dirname(os.path.abspath(__file__))

# Update the following lines:
#  - Assets folder within the NEW_NAME Building Block
NEW_NAME_ASSETS_PATH = os.path.join(BB_SOURCE_PATH, "assets")
#  - Container definition for NEW_NAME Building Block
NEW_NAME_CONTAINER = CONTAINER_PATH + "NEW_NAME.sif"  # TODO: Define your container




import argparse
import os
import json
import sys

from permedcoe.utils.user_arguments import Arguments


def parse_sys_argv():
    """ Parses the sys.argv.

    Args:
        None
    Returns:
        All arguments as namespace.
    """
    parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)         # noqa: E501
    parser.add_argument("-d", "--debug",
                        help="Enable debug mode. Overrides log_level",
                        action="store_true")
    parser.add_argument("-l", "--log_level",
                        help="Set logging level.",
                        choices=["debug", "info", "warning", "error", "critical"],                   # noqa: E501
                        default="error",
                        type=str)

    # Parent parser - includes all arguments which are common to all actions
    parent_parser = argparse.ArgumentParser(add_help=False,
                                            formatter_class=argparse.ArgumentDefaultsHelpFormatter)  # noqa: E501
    # Currently empty

    # Execute sub-parser
    subparsers = parser.add_subparsers(dest='action')
    parser_execute = subparsers.add_parser('execute',
                                           aliases=['x'],
                                           help='Execute a building block.',
                                           parents=[parent_parser],
                                           formatter_class=argparse.ArgumentDefaultsHelpFormatter)   # noqa: E501
    subparsers_execute = parser_execute.add_subparsers(dest='execute')
    parser_execute_bb = subparsers_execute.add_parser('building_block',
                                                      aliases=['bb'],
                                                      help='Execute a building block.',                        # noqa: E501
                                                      formatter_class=argparse.ArgumentDefaultsHelpFormatter)  # noqa: E501
    parser_execute_bb.add_argument(dest='name',
                                   type=str,
                                   help='Building Block to execute')
    __bb_execute_arguments__(parser_execute_bb)
    parser_execute_app = subparsers_execute.add_parser('application',
                                                       aliases=['app'],
                                                       help='Execute an application.',                          # noqa: E501
                                                       formatter_class=argparse.ArgumentDefaultsHelpFormatter)  # noqa: E501
    parser_execute_app.add_argument(dest='name',
                                    type=str,
                                    help='Application to execute')
    parser_execute_app.add_argument(dest='parameters',
                                    type=str,
                                    nargs='*',
                                    help='Application parameters')
    parser_execute_app.add_argument('-w', '--workflow_manager',
                                    type=str,
                                    choices=["none", "pycompss", "nextflow", "snakemake"],           # noqa: E501
                                    default="none",
                                    help='Workflow manager to use')
    parser_execute_app.add_argument('-f', '--flags',
                                    type=str,
                                    nargs='+',
                                    help='Workflow manager flags')
    # TODO: If provide support for application submission to Croupier
    # Submit sub-parser
    # parser_submit = subparsers.add_parser('submit',
    #                                       aliases=['s'],
    #                                       help='Submit an application.',
    #                                       parents=[parent_parser],
    #                                       formatter_class=argparse.ArgumentDefaultsHelpFormatter)    # noqa: E501
    # parser_submit.add_argument('-s', '--system',
    #                            dest='system',
    #                            type=str,
    #                            help='System where to submit')
    # parser_submit.add_argument(dest='name',
    #                            type=str,
    #                            help='Application name')
    # parser_submit.add_argument(dest='parameter',
    #                            type=str,
    #                            nargs='+',
    #                            help='Application parameter')

    # Templates
    parser_template = subparsers.add_parser('template',
                                            aliases=['t'],
                                            help='Shows an example of the requested template.',      # noqa: E501
                                            formatter_class=argparse.ArgumentDefaultsHelpFormatter)  # noqa: E501
    subparser_template = parser_template.add_subparsers(dest='template')
    parser_template_bb = subparser_template.add_parser('building_block',
                                                       aliases=['bb'],
                                                       help='Create a building block template.',                # noqa: E501
                                                       formatter_class=argparse.ArgumentDefaultsHelpFormatter)  # noqa: E501
    parser_template_bb.add_argument(dest='name',
                                    type=str,
                                    help='Building Block to create')
    parser_template_app = subparser_template.add_parser('application',
                                                        aliases=['app'],
                                                        help='Create an application template.',                  # noqa: E501
                                                        formatter_class=argparse.ArgumentDefaultsHelpFormatter)  # noqa: E501
    parser_template_app.add_argument(dest='name',
                                     type=str,
                                     help='Application to create')
    parser_template_app.add_argument('-t', '--type',
                                     choices=["all", "pycompss", "nextflow", "snakemake"],  # noqa: E501
                                     default="all",
                                     type=str,
                                     help='Application type.')

    # Check if the user does not include any argument
    if len(sys.argv) < 2:
        #  Show the usage
        print(parser.print_usage())
        sys.exit(1)

    arguments = parser.parse_args()

    # Check if the user does not include any argument after the action
    if sys.argv[-1] in ["execute", "x", "template", "t"]:
        #  Show the usage
        action = sys.argv[-1]
        try:
            print(subparsers.choices[action].print_usage())
        except KeyError:
            print(parser.print_usage())
        sys.exit(1)

    return arguments


def single_bb_sysarg_parser(bb_arguments):
    """ Parses the sys.argv.

    Args:
        bb_arguments: Building block arguments.
    Returns:
        Parsed arguments
    """
    if bb_arguments:
        # Building block detailed parser
        parser = argparse.ArgumentParser(
            description=bb_arguments.get_description()
        )
        __bb_specific_arguments__(parser, bb_arguments)
    else:
        # Old-school argument parser
        parser = argparse.ArgumentParser()
        __bb_execute_arguments__(parser)
    __bb_common_arguments__(parser)
    args = parser.parse_args()

    if hasattr(args, "mode") and args.mode == None:
        #  Show the usage
        print(parser.print_help())
        print("Please, specify the appropriate parameters")
        sys.exit(1)
    elif bb_arguments:
        # Check if input file and directory arguments exist
        __bb_arguments_checks__(args, bb_arguments)
    else:
        # Check if the user does not include input and output
        if not args.input or not args.output:
            #  Show the usage
            print(parser.print_help())
            print("Please, specify input and output")
            sys.exit(1)

    return args


def __bb_arguments_checks__(args, bb_arguments):
    """ Check if input file and directory arguments exist.

    Args:
        args (argparsed): Parsed arguments.
        bb_arguments (dict): BB arguments information.
    """
    if hasattr(args, "mode"):
        mode = args.mode
    else:
        mode = "default"
    arguments = bb_arguments.get_arguments()[mode]
    print(arguments)
    input_arguments = arguments.get_inputs()
    issues = []
    for k, v in input_arguments.items():
        check = v.get_check()
        to_check = vars(args)[k]
        if check == "file":
            if not os.path.isfile(to_check):
                issues.append("ERROR: Argument %s for file %s does not exist." % (k, to_check))
        if check == "folder":
            if not os.path.isdir(to_check):
                issues.append("ERROR: Argument %s for directory %s does not exist." % (k, to_check))
    output_arguments = arguments.get_outputs()
    if issues:
        for message in issues:
            print(message)
        raise Exception("ERROR: Wrong or missing argument/s.")


def __bb_specific_arguments__(parser, bb_arguments):
    """ BB specific arguments for Building block execute.

    Args:
        parser (parser): Parser to append arguments.
        bb_arguments (dict): BB arguments information.
    """
    arguments = bb_arguments.get_arguments()
    if len(arguments) == 1:
        # Only default mode
        inputs = arguments["default"].get_inputs()
        for param_name, param in inputs.items():
            help_msg = __get_help_message__("INPUT", param)
            parser_inner = parser.add_argument("--%s" % param_name,
                                               help=help_msg,
                                               type=param.get_type(),
                                               required=True)
        outputs = arguments["default"].get_outputs()
        for param_name, param in outputs.items():
            help_msg = __get_help_message__("OUTPUT", param)
            parser_inner = parser.add_argument("--%s" % param_name,
                                               help=help_msg,
                                               type=param.get_type(),
                                               required=True)
    else:
        # More than one mode
        subparser = parser.add_subparsers(dest="mode")
        for k, v in arguments.items():
            parser_mode = subparser.add_parser(k)
            inputs = v.get_inputs()
            for param_name, param in inputs.items():
                help_msg = __get_help_message__("INPUT", param)
                parser_inner_mode = parser_mode.add_argument("--%s" % param_name,
                                                             help=help_msg,
                                                             type=param.get_type(),
                                                             required=True)
            outputs = v.get_outputs()
            for param_name, param in outputs.items():
                help_msg = __get_help_message__("OUTPUT", param)
                parser_inner_mode = parser_mode.add_argument("--%s" % param_name,
                                                             help=help_msg,
                                                             type=param.get_type(),
                                                             required=True)


def __get_help_message__(param_direction, param):
    """ Generate help message.

    Args:
        param_type (str): If the parameter is INPUT or OUTPUT.
        param (Argument): Parameter argument object.
    Returns:
        The message associated for the given parameter.
    """
    if param.get_check() in ["file", "folder"]:
        param_type = "%s (%s)" % (param.get_type().__name__, param.get_check())
    else:
        param_type = param.get_type().__name__
    help_msg = "(%s - %s) %s" % (
        param_direction,
        param_type,
        param.get_description()
    )
    return help_msg



def __bb_execute_arguments__(parser):
    """ Add old-school arguments for Building block execute.

    Args:
        parser (parser): Parser to append arguments
    """
    parser.add_argument("-i", "--input",
                        help="Input file/s or directory path/s",
                        nargs="+",
                        type=str)
    parser.add_argument("-o", "--output",
                        help="Output file/s or directory path/s",
                        nargs="+",
                        type=str)


def __bb_common_arguments__(parser):
    """ Add common arguments for Building block execute.

    Args:
        parser (parser): Parser to append arguments
    """
    parser.add_argument("-c", "--config",
                    help="(CONFIG) Configuration file path",
                    type=str)
    parser.add_argument("-d", "--debug",
                        help="Enable Building Block debug mode. Overrides log_level",        # noqa: E501
                        action="store_true")
    parser.add_argument("-l", "--log_level",
                        help="Set logging level",
                        choices=["debug", "info", "warning", "error", "critical"],           # noqa: E501
                        type=str)
    parser.add_argument("--tmpdir",
                        help="Temp directory to be mounted in the container",
                        type=str)
    parser.add_argument("--processes",
                        help="Number of processes for MPI executions",
                        type=int)
    parser.add_argument("--gpus",
                        help="Requirements for GPU jobs",
                        type=int)
    parser.add_argument("--memory",
                        help="Memory requirement",
                        type=int)
    parser.add_argument("--mount_points",
                        help="Comma separated alias:folder to be mounted in the container",  # noqa: E501
                        type=str)
    # Hidden flag for advanced users
    parser.add_argument("--disable_container",
                        help=argparse.SUPPRESS,
                        action="store_true")


def load_parameters_from_json(parameters_file):
    """ Load the parameters defined in the given json file.

    Args:
        parameters_file (str): File containing the parameters (json format).
    Returns:
        Loaded arguments.
    """
    with open(parameters_file, "r") as params_fd:
        raw_params = json.load(params_fd)
    # Now build the args object from raw_params
    short_description = raw_params["short_description"]
    long_description = raw_params["long_description"]
    use_description = raw_params["use_description"]
    parameters = raw_params["parameters"]

    arguments = Arguments()
    if use_description == "short":
        arguments.set_description(short_description)
    else:
        arguments.set_description(long_description)
    # just_one_mode avoids to use default as keyword
    just_one_mode = len(parameters) == 1
    for mode, params in parameters.items():
        for param in params:
            p_type = param["type"]
            p_name = param["name"]
            p_format, p_check = __convert_format__(param["format"])
            p_description = param["description"]

            if p_type == "input":
                if just_one_mode:
                    arguments.add_input(name=p_name,
                                        type=p_format,
                                        description=p_description,
                                        check=p_check)
                else:
                    arguments.add_input(name=p_name,
                                        type=p_format,
                                        description=p_description,
                                        check=p_check,
                                        mode=mode)
            elif p_type == "output":
                if just_one_mode:
                    arguments.add_output(name=p_name,
                                        type=p_format,
                                        description=p_description)
                else:
                        arguments.add_output(name=p_name,
                                            type=p_format,
                                            description=p_description,
                                            mode=mode)
            else:
                raise Exception("Unexpected parameter type %s (supported input | output)" % str(p_type))
    return arguments


def __convert_format__(format):
    """Convert format to the actual type.

    Args:
        format (str): Input format.
    Returns:
        The supported format type and format to be checked.
    """
    if format == "str":
        return str, str
    elif format == "int":
        return int, int
    elif format == "float":
        return float, float
    elif format == "bool":
        return bool, bool
    elif format in ["file", "folder"]:
        return str, format
    else:
        raise Exception("Unsupported parameter type")



import os
from permedcoe.core.constants import SEPARATOR

DO_NOT_PARSE = (".pyc", ".def", ".sif")
PARSING_KEY = "NEW_NAME"
PARSING_PATH = "/PATH/TO/"


def adapt_name(name, path):
    """ Replace recursively into the given path they keyword with name.
    Like a set recursively.

    Args:
        name (str): Name to personalize the files.
        path (str): Path to find the files.
    """
    template_path = path + "/"
    for directory_name, dirs, files in os.walk(path):
        for file_name in files:
            if not file_name.endswith(DO_NOT_PARSE):
                file_path = os.path.join(directory_name, file_name)
                with open(file_path) as f:
                    s = f.read()
                s = s.replace(PARSING_KEY, name)
                s = s.replace(PARSING_PATH, template_path)
                with open(file_path, "w") as f:
                    f.write(s)


def rename_folder(name, path):
    """ Adapt the building block folder name.

    Args:
        name (str): Name to personalize the folder name.
        path (str): Path to find the files.
    """
    source = os.path.join(path, "src", "bb")
    destination = os.path.join(path, "src", name)
    os.rename(source, destination)


def show_todo(path):
    """ Show on the screen all to do messages.

    Args:
        path (str): Artifact path.
    """
    print(SEPARATOR)
    print("To be completed:")
    print()
    for directory_name, dirs, files in os.walk(path):
        for file_name in files:
            if not file_name.endswith(DO_NOT_PARSE):
                file_path = os.path.join(directory_name, file_name)
                __show_work__(file_path)
    print(SEPARATOR)


def __show_work__(file_path):
    """ Show the TODO messages of a given set of lines.

    Args:
        file_path (str): File to be analyzed.
    """
    with open(file_path) as f:
        lines = f.readlines()
    position = 0
    for line in lines:
        if "TODO" in line:
            _, message = line.split("#")
            print("- %s:(%s):\t%s" % (str(os.path.basename(file_path)),
                                      str(position),
                                      str(message).strip()))
        position += 1



#!/usr/bin/python3

from permedcoe.utils.arguments import parse_sys_argv \
    as __parse_sys_argv__
from permedcoe.core.functions import execute_building_block \
    as __execute_building_block__
from permedcoe.core.functions import execute_application \
    as __execute_application__
from permedcoe.core.functions import create_template \
    as __create_template__


def main():
    """
    Main entry point for the "permedcoe" executable.
    """
    # Parse sys.argv
    arguments = __parse_sys_argv__()
    debug = True if arguments.debug else False
    # Action selector
    if arguments.action in ["execute", "x"]:
        if arguments.execute in ["building_block", "bb"]:
            if debug:
                print("Executing Building Block")
            __execute_building_block__(arguments)
        if arguments.execute in ["application", "app"]:
            if debug:
                print("Executing Application")
            __execute_application__(arguments)
    if arguments.action in ["submit", "s"]:
        if debug:
            print("Submit Application")
        raise NotImplementedError
    if arguments.action in ["template", "t"]:
        if arguments.template in ["building_block", "bb"]:
            app_type = None
            if debug:
                print("Creating building block template")
        else:
            app_type = arguments.type
            if debug:
                print("Creating application template")
        __create_template__(arguments.debug,
                            arguments.log_level,
                            arguments.template,
                            arguments.name,
                            app_type)


if __name__ == "__main__":
    main()



import sys
import logging
import subprocess
from permedcoe.core.constants import SEPARATOR

DECODING_FORMAT = 'utf-8'


def command_runner(cmd):
    """Run the command defined in the cmd list.

    Args:
        cmd (list[str]): Command to execute as list.

    Raises:
        Exception: Exit code != 0
    """
    logging.debug("Executing: %s" % str(cmd))
    p = subprocess.Popen(cmd,
                         stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE)
    stdout, stderr = p.communicate()   # blocks until cmd is done
    stdout = stdout.decode(DECODING_FORMAT)
    stderr = stderr.decode(DECODING_FORMAT)
    return_code = p.returncode
    logging.debug("Exit code: %s" % str(return_code))

    print(SEPARATOR, flush=True)
    print("------------ STDOUT ------------",
          flush=True)
    print(stdout, flush=True)
    if stderr:
        print("------------ STDERR ------------",
              file=sys.stderr, flush=True)
        print(stderr, file=sys.stderr, flush=True)

    print(SEPARATOR, flush=True)
    if return_code != 0:
        print("Exit code: %s != 0" % str(return_code),
              file=sys.stderr, flush=True)
        exit(return_code)



"""
NOTE: Do not modify this file unless you want to change the name
      of the main.py file. Then it is necessary to adapt the import.
"""

import os
from permedcoe import invoker
from NEW_NAME.main import invoke
from NEW_NAME.definitions import BB_SOURCE_PATH


def main():
    invoker(invoke, os.path.join(BB_SOURCE_PATH, "definition.json"))


if __name__ == "__main__":
    main()



import yaml
import logging

from permedcoe.utils.log import init_logging
from permedcoe.utils.environ import set_environment
import permedcoe.core.environment as cmd_flags


def preprocessing(arguments):
    # Parse configuration file
    if arguments.config:
        with open(arguments.config, "r") as config_fd:
            cfg = yaml.safe_load(config_fd)
    else:
        cfg = None
    # Initialize logging
    init_logging(arguments.debug, arguments.log_level)
    # Global environment flags from command line
    cmd_flags.DEBUG = arguments.debug
    cmd_flags.DISABLE_CONTAINER = arguments.disable_container
    # Export variables
    set_environment(arguments.tmpdir,
                    arguments.processes,
                    arguments.gpus,
                    arguments.memory,
                    arguments.mount_points)
    # Show arguments
    logging.debug("Arguments:")
    for param, value in arguments.__dict__.items():
        logging.debug("\t - " + str(param) + " : " + str(value))
    return cfg



"""
This file provides the PerMedBB class which implements the functionalities
to interact with the container infrastructure (Singularity).
"""

import os
import logging


class PerMedBB(object):

    def __init__(self, img_path, mpi_runner, exe_path,
                 computing_nodes, computing_units,
                 mount_paths, user_mount_paths,
                 env_vars, flags):
        """ Constructor

        Args:
            img_path (str): Container path.
            mpi_runner (str): MPI runner. (default=None)
            exe_path (str): Executable path.
            computing_nodes (int): Number of compute nodes needed.
            computing_units (int): Number of compute units needed (cores).
            mount_paths (list[str]): List of folders to be mounted.
            user_mount_paths (list[str]): User defined mount paths.
            env_vars (dict{name: value}): Environment variables to delegate.
            flags (dict{name: value}): Flags for the container engine.
        """
        self.img_path = img_path
        self.exe_path = exe_path
        self.flags = flags

        self.sing_command_comp = {}
        self.sing_command_comp["base"] = "singularity --silent"
        self.sing_command_comp["action"] = "exec"
        self.sing_command_comp["action_flags"] = "--contain --cleanenv --pwd " + os.getcwd()  # noqa: E503
        self.sing_command_comp["sif"] = img_path
        if mpi_runner:
            # MPI execution within the container
            self.sing_command_comp["exe"] = [mpi_runner,
                                             "-np",
                                             self.computing_units,
                                             exe_path]
        else:
            # Single binary execution
            self.sing_command_comp["exe"] = exe_path
        self.sing_command_comp["mounts"] = ""
        for path in mount_paths:
            self.add_bind(path, path)
        if user_mount_paths:
            user_mounts = user_mount_paths.split(",")
            for mount in user_mounts:
                s, t = mount.split(":")
                self.add_bind(s, t)
        self.sing_command_comp["envs"] = ""
        for var in env_vars:
            self.add_env(var)
        self.sing_command_comp["flags"] = flags

    def add_env(self, env_var):
        """ Small helper function to add an environment variable to
        singularity command line.

        Args:
            env_var (str): Environment variable.
        """
        if self.sing_command_comp["envs"] == "":
            self.sing_command_comp["envs"] = "--env {}".format(env_var)
        else:
            self.sing_command_comp["envs"] = \
                self.sing_command_comp["envs"] + ",{}".format(env_var)

    def add_bind(self, s, t):
        """ Small helper function to add a bind point to singularity command line.

        Args:
            s (str): Source folder
            t (str): Destination folder
        """
        if self.sing_command_comp["mounts"] == "":
            self.sing_command_comp["mounts"] = "-B {}:{}".format(s, t)
        else:
            self.sing_command_comp["mounts"] = \
                self.sing_command_comp["mounts"] + ",{}:{}".format(s, t)

    def launch(self, shell=False, run_in_container=True):
        """ Executes the binary into the singularity container.

        Args:
            shell (bool, optional): Action shell. Defaults to False.
            run_in_container (bool, optional): Launch execution in container.
        """
        command = ""
        if run_in_container:
            order = ["base",
                     "action",
                     "action_flags",
                     "mounts",
                     "envs",
                     "sif",
                     "exe",
                     "flags"]
            if shell:
                self.sing_command_comp["action"] = "shell"
        else:
            order = ["exe",
                     "flags"]
        scc = self.sing_command_comp
        for c in order:
            if isinstance(scc[c], list):
                for elem_scc in scc[c]:
                    if isinstance(elem_scc, list):
                        command = command + " " + " ".join(elem_scc)
                    else:
                        command = command + " " + str(elem_scc)
            else:
                command = command + " " + scc[c]
        logging.info("Launching the command: %s" % command)
        os.system(command)



"""Private functions used by the building blocks."""

import os


def get_container_path():
    """Retrieve the PerMedCoE container images path.

    :return: The value of the PERMEDCOE_IMAGES environment variable.
    :raises Exception: If the "PERMEDCOE_IMAGES" is not defined.
    """
    # Container definition environment variable
    CONTAINER_PATH_VN = "PERMEDCOE_IMAGES"
    if CONTAINER_PATH_VN in os.environ:
        return os.environ[CONTAINER_PATH_VN]
    else:
        raise Exception("Please define %s environment variable with the path." % CONTAINER_PATH_VN)


CONTAINER_PATH = get_container_path()


def get_computing_units():
    """Retrieve the computing units required for the tasks.

    :return: The value of the COMPUTING_UNITS environment variable as integer.
    """
    # Computing units for the tasks
    COMPUTING_UNITS_VN = "COMPUTING_UNITS"
    if COMPUTING_UNITS_VN in os.environ:
        return int(os.environ[COMPUTING_UNITS_VN])
    else:
        return 1


COMPUTING_UNITS = get_computing_units()



class Argument:
    def __init__(self, type, description, check=None):
        self.type = type
        self.description = description
        self.check = check

    def get_type(self):
        return self.type

    def get_description(self):
        return self.description

    def get_check(self):
        return self.check

    def __str__(self) -> str:
        args_str = "\t- Type: %s\n" % self.type
        args_str += "\t- Description: %s\n" % self.description
        if self.check:
            args_str += "\t- Check: %s\n" % str(self.check)
        return args_str


class ArgumentDirections:
    def __init__(self):
        self.inputs = {}
        self.outputs = {}

    def add_input(self, name, input):
        self.inputs[name] = input

    def add_output(self, name, output):
        self.outputs[name] = output

    def get_inputs(self):
        return self.inputs

    def get_outputs(self):
        return self.outputs

    def __str__(self) -> str:
        args_str = ""
        for k, v in self.inputs.items():
            args_str += "- Input: %s\n" % k
            args_str += "%s" % v
        for k, v in self.outputs.items():
            args_str += "- Output: %s\n" % k
            args_str += "%s" % v
        return args_str


class Arguments:

    def __init__(self):
        self.arguments = {}
        self.arguments["default"] = ArgumentDirections()
        self.description = "Undefined description."

    def set_description(self, description):
        self.description = description

    def add_input(self, name, type, description, check=None, mode="default"):
        if not mode in self.arguments:
            self.arguments[mode] = ArgumentDirections()
        self.arguments[mode].add_input(name, Argument(type, description, check))

    def add_output(self, name, type, description, mode="default"):
        if not mode in self.arguments:
            self.arguments[mode] = ArgumentDirections()
        self.arguments[mode].add_output(name, Argument(type, description))

    def get_arguments(self) -> dict:
        return self.arguments

    def get_description(self):
        return self.description

    def __str__(self) -> str:
        args_str = ""
        for k, v in self.arguments.items():
            args_str += "Mode: %s\n" % k
            args_str += "Arguments: %s\n" % v
        return args_str



from permedcoe.utils.arguments import single_bb_sysarg_parser as __bb_parser__
from permedcoe.utils.arguments import load_parameters_from_json as __bb_param_loader__
from permedcoe.utils.preproc import preprocessing as __preprocessing__
from permedcoe.utils.log import init_logging as __init_logging__
from permedcoe.utils.environ import get_environment as __get_environment__
import permedcoe.core.environment as __cmd_flags__


def get_environment():
    """ Retrieve the environment variables set on call as dictionary.

    Returns:
        dict: Dictionary containing the environment set on call.
    """
    return __get_environment__()


def set_debug(level=False):
    """ Sets the global debug variable

    Args:
        debug: True to enable debug. False otherwise.
    Returns:
        None
    """
    __init_logging__(level)


def invoker(function, arguments_info=None) -> None:
    """ Parse the input parameters (from sys.argv) and then invoke the
    given BB function.

    Args:
        function (function): Building block function to invoke.
        arguments_info (function): Building block arguments information.
    Returns:
        None
    """
    if arguments_info:
        # Grab the BB arguments info to tune the argument parser
        if isinstance(arguments_info, str):
            # Use the parameters defined in the given file
            bb_arguments = __bb_param_loader__(arguments_info)
        else:
            # Use the given arguments
            bb_arguments = arguments_info()
        old_school_args = False
    else:
        # If set to none, use old-school inputs and outputs parameters
        bb_arguments = None
        old_school_args = True
    arguments = __bb_parser__(bb_arguments)
    if arguments.debug:
        print("Building Block arguments:\n%s" % str(bb_arguments))
    # Set execution related conditions
    __cmd_flags__.DEBUG = arguments.debug
    __cmd_flags__.DISABLE_CONTAINER = arguments.disable_container
    set_debug(arguments.debug)
    # Preprocess
    cfg = __preprocessing__(arguments)
    if old_school_args:
        # Grab input and output
        in_path = arguments.input
        out_path = arguments.output
        # Building block invocation
        function(in_path, out_path, cfg)
    else:
        # Building block invocation
        function(arguments, cfg)



# Always prefer setuptools over distutils
from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

# Get the long description from the README file
long_description = (here / "README.md").read_text(encoding="utf-8")

setup(
    name="NEW_NAME",
    version="0.0.1",
    description="This package provides the NEW_NAME Building Block",
    long_description_content_type="text/markdown",
    long_description=long_description,
    url="TO_BE_DEFINED",
    author="Author",
    author_email="author@example.com",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3 :: Only",
        "Development Status :: 1 - Planning",
        "Intended Audience :: Science/Research",
        "Operating System :: Unix",
        "Operating System :: POSIX :: Linux",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "Topic :: Scientific/Engineering :: Medical Science Apps.",
    ],
    keywords="BuildingBlock, NEW_NAME",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    python_requires=">=3.6, <4",
    install_requires=["permedcoe>=0.0.6"],
    extras_require={
        "dev": ["check-manifest"],
    },
    # If there are data files included in your packages that need to be
    # installed, specify them here.
    package_data={
        "NEW_NAME": ["assets/*",
                     "definition.json"],
    },
    entry_points={
        "console_scripts": [
            "NEW_NAME=NEW_NAME.__main__:main",
        ],
    },
    project_urls={
        "Bug Reports": "TO_BE_DEFINED",
        "Source": "TO_BE_DEFINED",
    },
)





#!/usr/bin/python3

import sys

# To set building block debug mode
from permedcoe import set_debug
# TODO: Import the desired building blocks entry points and use invoke or any other function.
from my_building_block import invoke
from my_building_block import building_block_task


def main():
    # Enable set_debug to have a more verbose output
    set_debug(False)
    # Sample application:
    print("Sample python application using my_building_block BB")
    # Get parameters
    input_file = str(sys.argv[1])
    output_file = str(sys.argv[2])
    conf = {}  # conf is empty since it is not used by my_building_block
    # Building Block invocation
    building_block_task(input_file=input_file,
                        output_file=output_file)
    # Alternative Building Block invocation (uning invoke method)
    # invoke(input_file,
    #        output_file)


if __name__ == "__main__":
    main()



import os
import importlib
import logging
import pathlib
import shutil

from permedcoe.utils.preproc import preprocessing
from permedcoe.utils.log import init_logging
from permedcoe.utils.executor import command_runner
from permedcoe.utils.artifact import adapt_name
from permedcoe.utils.artifact import rename_folder
from permedcoe.utils.artifact import show_todo


BUILDING_BLOCK_LABELS = ("building_block", "bb")
APPLICATION_LABELS = ("application", "app")


def execute_building_block(arguments):
    """ Execute the requested building block.

    Args:
        arguments (Namespace): System arguments.
    """
    # Grab input and output
    in_path = arguments.input
    out_path = arguments.output
    # Preprocess
    cfg = preprocessing(arguments)
    # Building block invocation
    building_block = arguments.name
    logging.info("Executing Building Block: %s" % str(building_block))
    bb_module = importlib.import_module(building_block)
    invoke = getattr(bb_module, "invoke")
    invoke(in_path, out_path, cfg)


def execute_application(arguments):
    """ Execute the requested building block.

    Args:
        arguments (Namespace): System arguments.
    """
    # Application related arguments
    app_name = arguments.name
    app_parameters = arguments.parameters
    workflow_manager = arguments.workflow_manager
    workflow_flags = arguments.flags
    # Init logging
    init_logging(arguments.debug, arguments.log_level)
    logging.info("Executing Application: %s" % str(app_name))
    logging.info("Parameters: %s" % str(app_parameters))
    logging.info("Workflow manager: %s" % str(workflow_manager))
    logging.info("Workflow flags: %s" % str(workflow_flags))
    # Workflow manager selector
    command = []
    if workflow_manager == "pycompss":
        command.append("runcompss")
    elif workflow_manager == "nextflow":
        command.append("nextflow")
    elif workflow_manager == "snakemake":
        command.append("snakemake")
    else:
        command.append("python3")
    if workflow_flags:
        for flag in workflow_flags:
            command += flag.split()
    if workflow_manager == "snakemake":
        command.append("--snakefile")
    command.append(app_name)
    if app_parameters:
        for parameter in app_parameters:
            command += parameter.split()
    command_runner(command)


def create_template(debug, log_level, artifact, name, app_type=None):
    """ Creates a building block or application template.

    Args:
        artifact (str): Artifact (building block or application)
        name (str): Artifact name
        type (str): Specific type of application (ignored for building blocks)

    Raises:
        Exception: Unsupported artifact
    """
    # Init logging
    init_logging(debug, log_level)
    # Prepare destination
    current_path = pathlib.Path().absolute()
    if os.path.exists(os.path.join(current_path, name)):
        print("Can not create template. A folder with name %s already exists." % str(name))  # noqa: E501
        exit(1)
    # Prepare source
    egg_path = pathlib.Path(__file__).parent.parent.absolute()
    source_path = os.path.join(egg_path, "templates")
    # Choose what to extract to this folder
    if artifact in BUILDING_BLOCK_LABELS:
        print("Creating Building Block template")
        skeleton_path = os.path.join(source_path,
                                     "building_block",
                                     "skeleton_BB")
    elif artifact in APPLICATION_LABELS:
        print("Creating Application template")
        app_path = os.path.join(source_path, "application")
        if app_type == "pycompss":
            skeleton_path = os.path.join(app_path, "PyCOMPSs")
        elif app_type == "nextflow":
            skeleton_path = os.path.join(app_path, "NextFlow")
        elif app_type == "snakemake":
            skeleton_path = os.path.join(app_path, "SnakeMake")
        else:
            skeleton_path = app_path
    else:
        raise Exception("Unrecognized template type: %s" % str(artifact))
    # Copy from sources to destination
    destination_path = os.path.join(current_path, name)
    logging.debug("Copying artifact to: %s" % str(destination_path))
    shutil.copytree(skeleton_path, destination_path)
    # Adapt name into the files
    logging.debug("Adapting name: %s to the artifact" % str(name))
    adapt_name(name, destination_path)
    # Adapt folder name if needed
    if artifact in BUILDING_BLOCK_LABELS:
        # Rename folder
        logging.debug("Adapting building block source folder name")
        rename_folder(name, destination_path)
        logging.debug("Artifact created")
    # Show TODO messages
    show_todo(destination_path)



# Always prefer setuptools over distutils
from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

# Get the long description from the README file
long_description = (here / "README.md").read_text(encoding="utf-8")

setup(
    name="permedcoe",
    version="0.0.6",
    description="This package provides the common interface for the Building Blocks.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/PerMedCoE/permedcoe",
    author="PerMedCoE Project",
    author_email="infoPerMedCoE@bsc.es",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: Apache Software License",
        "Intended Audience :: Science/Research",
        "Operating System :: Unix",
        "Operating System :: POSIX :: Linux",
        "Operating System :: MacOS",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3 :: Only",
        "Environment :: Console",
    ],
    keywords="PerMedCoE",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    python_requires=">=3.6, <4",
    install_requires=["pyyaml"],
    extras_require={
        "dev": ["check-manifest"],
    },
    # If there are data files included in your packages that need to be
    # installed, specify them here.
    package_data={
        "permedcoe": ["templates/*",
                      "templates/application/*",
                      "templates/application/NextFlow/*",
                      "templates/application/PyCOMPSs/*",
                      "templates/application/SnakeMake/*",
                      "templates/building_block/*",
                      "templates/building_block/skeleton_BB/*",
                      "templates/building_block/skeleton_BB/container/*",
                      "templates/building_block/skeleton_BB/src/*",
                      "templates/building_block/skeleton_BB/src/bb/*",
                      "templates/building_block/skeleton_BB/src/bb/assets/*"],
    },
    entry_points={
        "console_scripts": [
            "permedcoe=permedcoe.__main__:main",
        ],
    },
    project_urls={
        "Bug Reports": "https://github.com/PerMedCoE/permedcoe/issues",
        "Source": "https://github.com/PerMedCoE/permedcoe",
    },
)



"""
This file implements the decorators to be used in the building blocks,
which wrap the container, binary and execution actions.
"""

import os
import sys
import inspect
import subprocess
import logging
from collections import OrderedDict

from permedcoe.core.building_block import PerMedBB
import permedcoe.core.environment as cmd_flags
from permedcoe.utils.exceptions import ContainerImageException

# Environment variable names
from permedcoe.core.constants import PERMEDCOE_TMPDIR
from permedcoe.core.constants import PERMEDCOE_PROCESSES
from permedcoe.core.constants import PERMEDCOE_GPUS
from permedcoe.core.constants import PERMEDCOE_MEMORY
from permedcoe.core.constants import PERMEDCOE_MOUNT_POINTS
from permedcoe.core.constants import SEPARATOR


# ################################################################# #
# ########################## DECORATORS ########################### #
# ################################################################# #

class Container(object):
    """
    Container class (decorator style)

    Gets the container related information.
    """

    def __init__(self, *args, **kwargs):
        # Decorator parameters
        self.args = args
        self.kwargs = kwargs

    def __call__(self, f):
        def wrapped_f(*args, **kwargs):
            # args and kwargs are the function invocation parameters
            # Delegate the needed info to the task through the **kwargs.
            if "engine" in self.kwargs:
                kwargs["engine"] = self.kwargs["engine"]
            else:
                # Default engine if not specified
                kwargs["engine"] = "SINGULARITY"
            kwargs["image"] = self.kwargs["image"]
            return f(*args, **kwargs)
        return wrapped_f


class Constraint(object):
    """
    Dummy Constraint class (decorator style)
    """

    def __init__(self, *args, **kwargs):
        # Decorator parameters
        self.args = args
        self.kwargs = kwargs

    def __call__(self, f):
        def wrapped_f(*args, **kwargs):
            # args and kwargs are the function invocation parameters
            # Delegate the needed info to the task through the **kwargs.
            kwargs["computing_units"] = self.kwargs["computing_units"]
            return f(*args, **kwargs)
        return wrapped_f


class Mpi(object):
    """
    Mpi class (decorator style)

    Gets the MPI related information.
    """

    def __init__(self, *args, **kwargs):
        # Decorator parameters
        self.args = args
        self.kwargs = kwargs

    def __call__(self, f):
        def wrapped_f(*args, **kwargs):
            # args and kwargs are the function invocation parameters
            # Delegate the needed info to the task through the **kwargs.
            kwargs["runner"] = self.kwargs["runner"]
            kwargs["binary"] = self.kwargs["binary"]
            kwargs["computing_nodes"] = self.kwargs["computing_nodes"]  # noqa TODO: ignored since it requires node names
            if "environment" in self.kwargs:
                kwargs["environment"] = self.kwargs["environment"]
            else:
                kwargs["environment"] = []
            return f(*args, **kwargs)
        return wrapped_f


class Binary(object):
    """
    Binary class (decorator style)

    Gets the binary related information.
    """

    def __init__(self, *args, **kwargs):
        # Decorator parameters
        self.args = args
        self.kwargs = kwargs

    def __call__(self, f):
        def wrapped_f(*args, **kwargs):
            # args and kwargs are the function invocation parameters
            # Delegate the needed info to the task through the **kwargs.
            kwargs["binary"] = self.kwargs["binary"]
            if "environment" in self.kwargs:
                kwargs["environment"] = self.kwargs["environment"]
            else:
                kwargs["environment"] = []
            return f(*args, **kwargs)
        return wrapped_f


class Task(object):
    """
    Task class (decorator style) that defines the behaviour of a
    building block execution.
    Takes information from the higher level decorators (@container
    and @binary), does the necessary path transformations from
    relative to absolute, then instantiates a PerMed_BB instance
    with all this information and launches its execution.
    """

    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs

    def __call__(self, f):
        def wrapped_f(*args, **kwargs):
            # To dummy task:
            # return f(*args, **kwargs)
            # Instead, takes all learnt from previous decorators, and acts:
            # Deploys the container and executes the binary

            run_in_container = True
            if "engine" not in kwargs or cmd_flags.DISABLE_CONTAINER:
                # The @container has not been defined, so
                # disable running in container
                run_in_container = False

            # Pop the info from upper decorators:
            if run_in_container:
                self.engine = kwargs.pop("engine")
                self.image = kwargs.pop("image")
            else:
                self.engine = None
                self.image = None
            self.runner = kwargs.pop("runner", None)
            self.binary = kwargs.pop("binary")
            self.computing_nodes = kwargs.pop("computing_nodes", 1)
            self.computing_units = kwargs.pop("computing_units", 1)
            self.env_vars = kwargs.pop("environment")
            logging.debug(SEPARATOR)
            if run_in_container:
                logging.debug("Container engine          : %s" % self.engine)
                logging.debug("Container image           : %s" % self.image)
            logging.debug("Container runner          : %s" % self.runner)
            logging.debug("Container binary          : %s" % self.binary)
            logging.debug("Container computing_nodes : %s" % self.computing_nodes)  # noqa: E503
            logging.debug("Container computing_units : %s" % self.computing_units)  # noqa: E503
            logging.debug("Container env vars        : %s" % self.env_vars)
            logging.debug(SEPARATOR)

            # Parameters provided by the user:
            defaults = self.__get_defaults__(f)
            flags = OrderedDict(defaults, **kwargs)
            flags_list = list(flags.values())
            logging.debug("Provided flags: %s" % str(kwargs))
            # To increase debugging:
            # logging.debug("Default flags: %s" % str(defaults))
            logging.debug("User flags: %s" % str(flags_list))
            logging.debug(SEPARATOR)

            # Checks:
            #  - Container image
            if not os.path.isfile(self.image):
                raise ContainerImageException(self.image)

            # Look for mount paths:
            mount_paths, update_paths, user_mount_paths = \
                self.__find_mount_paths__(kwargs)

            if update_paths:
                # There are paths that need to be fixed in flags list
                for k, v in update_paths.items():
                    flags[k] = v
                flags_list = list(flags.values())

            if run_in_container:
                logging.debug("Mount paths:")
                for path in mount_paths:
                    logging.debug("- %s" % path)
                logging.debug(SEPARATOR)

            # Act:
            BB = PerMedBB(self.image,
                          self.runner,
                          self.binary,
                          self.computing_nodes,
                          self.computing_units,
                          mount_paths,
                          user_mount_paths,
                          self.env_vars,
                          flags_list)
            BB.launch(run_in_container=run_in_container)
        return wrapped_f

    def __find_mount_paths__(self, kwargs):
        """ Looks for mount paths into the give input/output files/directories.

        Args:
            kwargs (dict): Keyword dictionary (invocation parameters)

        Returns:
            list: List with the paths to be mounted.
            dict: Dictionary with the flags to be updated
                  (from relative to absolute).
        """
        mount_paths = []
        update_paths = {}
        # Look into the invocation parameters
        for k, v in self.kwargs.items():
            path = ""
            if v == FILE_IN or v == FILE_OUT:
                # Remove file name and keep only the folder
                if isinstance(kwargs[k], list):
                    for element in kwargs[k]:
                        path, name = os.path.split(os.path.realpath(element))
                        update_paths[k] = os.path.join(path, name)
                else:
                    path, name = os.path.split(os.path.realpath(kwargs[k]))
                    update_paths[k] = os.path.join(path, name)
            elif v == DIRECTORY_IN or v == DIRECTORY_INOUT or v == DIRECTORY_OUT:  # noqa: E503
                if isinstance(kwargs[k], list):
                    for element in kwargs[k]:
                        if os.path.exists(element):
                            path = os.path.realpath(element)
                        elif v == DIRECTORY_OUT:
                            os.mkdir(element)
                            path = os.path.realpath(element)
                        else:
                            raise Exception("Input directory does not exist: %s" % str(element))  # noqa: E503
                else:
                    if os.path.exists(kwargs[k]):
                        path = os.path.realpath(kwargs[k])
                    elif v == DIRECTORY_OUT:
                        os.mkdir(kwargs[k])
                        path = os.path.realpath(kwargs[k])
                    else:
                        raise Exception("Input directory does not exist: %s" % str(element))  # noqa: E503
            else:
                raise Exception("Unexpected task tag found.")
            # Relative path to absolute
            mount_paths.append(path)
        # Look into the environment
        if PERMEDCOE_TMPDIR in os.environ:
            mount_paths.append(os.environ[PERMEDCOE_TMPDIR])
        user_mount_paths = None
        if PERMEDCOE_MOUNT_POINTS in os.environ:
            user_mount_paths = os.environ[PERMEDCOE_MOUNT_POINTS]
        # Removes duplicated
        if mount_paths:
            mount_paths = list(set(mount_paths))
        return mount_paths, update_paths, user_mount_paths

    @staticmethod
    def __get_defaults__(func):
        """ Retrieve the default parameters of the given function

        Args:
            func (function): Function to be inspected.

        Returns:
            OrderedDict: Contains the parameter name as key and its
                         default value as its value.
        """
        signature = inspect.signature(func)
        return OrderedDict({
            k: v.default
            for k, v in signature.parameters.items()
            if v.default is not inspect.Parameter.empty
        })


# Naming convention with lowercase
task = Task
binary = Binary
container = Container
mpi = Mpi
constraint = Constraint

# Parameter definitons
FILE_IN = "FILE_IN"
FILE_OUT = "FILE_OUT"
FILE_INOUT = "FILE_INOUT"
DIRECTORY_IN = "DIRECTORY_IN"
DIRECTORY_OUT = "DIRECTORY_OUT"
DIRECTORY_INOUT = "DIRECTORY_INOUT"
Type = "type"
StdIOStream = "StdIOStream"
STDIN = None
STDOUT = None
STDERR = None





class ContainerImageException(Exception):
    """
    Container image exception.
    """

    def __init__(self, image_path):
        super(ContainerImageException, self).__init__("Container image " +
                                                      image_path +
                                                      " NOT FOUND!")



import os

from permedcoe.core.constants import PERMEDCOE_TMPDIR
from permedcoe.core.constants import PERMEDCOE_PROCESSES
from permedcoe.core.constants import PERMEDCOE_GPUS
from permedcoe.core.constants import PERMEDCOE_MEMORY
from permedcoe.core.constants import PERMEDCOE_MOUNT_POINTS


def set_environment(tmpdir=None,
                    processes=None,
                    gpus=None,
                    memory=None,
                    mount_points=None):
    """ Sets the necessary environment variables from the given arguments.

    Args:
        tmpdir (str, optional): Temporary directory to be mounted.
                                Defaults to None.
        processes (int, optional): Number of processes.
                                   Defaults to None.
        gpus (int, optional): Number of gpus.
                              Defaults to None.
        memory (int, optional): Amount of memory.
                                Defaults to None.
        mount_points (str, optional): User defined mount points.
                                      Defaults to None.
    """
    if tmpdir:
        if os.path.exists(tmpdir):
            os.environ[PERMEDCOE_TMPDIR] = tmpdir
        else:
            raise Exception("TMPDIR: %s does not exist." % tmpdir)
    if processes:
        os.environ[PERMEDCOE_PROCESSES] = processes
    if gpus:
        os.environ[PERMEDCOE_GPUS] = gpus
    if memory:
        os.environ[PERMEDCOE_MEMORY] = memory
    if mount_points:
        os.environ[PERMEDCOE_MOUNT_POINTS] = mount_points


def get_environment():
    """ Retrieve the environment variables set on call as dictionary.

    Returns:
        dict: Dictionary containing the environment set on call.
    """
    env_vars = {}
    if PERMEDCOE_TMPDIR in os.environ:
        env_vars["tmpdir"] = os.environ[PERMEDCOE_TMPDIR]
    if PERMEDCOE_PROCESSES in os.environ:
        env_vars["processes"] = os.environ[PERMEDCOE_PROCESSES]
    if PERMEDCOE_GPUS in os.environ:
        env_vars["gpus"] = os.environ[PERMEDCOE_GPUS]
    if PERMEDCOE_MEMORY in os.environ:
        env_vars["memory"] = os.environ[PERMEDCOE_MEMORY]
    if PERMEDCOE_MOUNT_POINT in os.environ:
        env_vars["mount_points"] = os.environ[PERMEDCOE_MOUNT_POINT]
    return env_vars
